@@ -81,21 +81,22 @@ def test_create_raster_polygons_basic():
 	# build raster polygons from a simple 2x2 grid of lat/lon pixels
 	ds = xr.Dataset({'lat_bnds':(['lat','bnds'],np.array([[-0.5,0.5],[0.5,1.5]])),
 					 'lon_bnds':(['lon','bnds'],np.array([[-0.5,0.5],[0.5,1.5]]))},
-					coords={'lat':(['lat'],np.array([0,1])),
-							'lon':(['lon'],np.array([0,1])),
-							'bnds':(['bnds'],np.array([0,1]))})
+					coords={'lat':(['lat'],np.array([0.,1.])),
+							'lon':(['lon'],np.array([0.,1.])),
+							'bnds':(['bnds'],np.array([0.,1.]))})
 	pix_agg = create_raster_polygons(ds)
 
-	# Create comparison geodataframe (what it should come out to)
-	poly_test = {'lat':[np.array(v) for v in [0.,0.,1.,1.]],'lon':[np.array(v) for v in [0.,1.,0.,1.]],
+    # Create comparison geodataframe (what it should come out to)
+	poly_test = {'lat':np.array([0.,0.,1.,1.]),'lon':np.array([0.,1.,0.,1.]),
 						'geometry':[Polygon([(-0.5,-0.5),(-0.5,0.5),(0.5,0.5),(0.5,-0.5),(-0.5,-0.5)]),
 								   Polygon([(0.5, -0.5), (0.5, 0.5), (1.5, 0.5), (1.5, -0.5), (0.5, -0.5)]),
 								   Polygon([(-0.5, 0.5), (-0.5, 1.5), (0.5, 1.5), (0.5, 0.5), (-0.5, 0.5)]),
 								   Polygon([(0.5, 0.5), (0.5, 1.5), (1.5, 1.5), (1.5, 0.5), (0.5, 0.5)])],
 						'pix_idx':[0,1,2,3]}
+    
 	poly_test = gpd.GeoDataFrame(poly_test,crs="EPSG:4326")
 
-	# Use gpd assert (there's a check_less_precise option for "almost equal", 
+    # Use gpd assert (there's a check_less_precise option for "almost equal", 
 	# but that doesn't seem necessary for this simple example)
 	gpdt.assert_geodataframe_equal(poly_test,pix_agg['gdf_pixels'])
 
@@ -326,8 +327,6 @@ def test_get_pixel_overlaps_gdf_wpreexisting_index(pix_agg=pix_agg):
 
 
 ##### aggregate() tests #####
-
-
 def test_aggregate_basic(ds=ds):
 	# Create polygon covering multiple pixels
 	gdf = {'name':['test'],
@@ -351,6 +350,29 @@ def test_aggregate_basic(ds=ds):
 	# Possibly worth examining more closely later
 	assert np.allclose([v for v in agg.agg.test.values],1.4999,rtol=1e-4)
 
+def test_aggregate_basic_wdotproduct(ds=ds):
+	# Create polygon covering multiple pixels, using the dot product option
+	gdf = {'name':['test'],
+				'geometry':[Polygon([(0,0),(0,1),(1,1),(1,0),(0,0)])]}
+	gdf = gpd.GeoDataFrame(gdf,crs="EPSG:4326")
+
+	# calculate the pix_agg variable tested above, to be used in the 
+	# tests below
+	pix_agg = create_raster_polygons(ds)
+
+
+	# Get pixel overlaps
+	wm = get_pixel_overlaps(gdf,pix_agg,impl='dot_product')
+
+	# Get aggregate
+	agg = aggregate(ds,wm,impl='dot_product')
+
+	# This requires shifting rtol to 1e-4 for some reason, in that 
+	# it's actually 1.499981, whereas multiplying out 
+	# np.sum(agg.agg.rel_area[0]*np.array([0,1,2,3]))gives 1.499963... 
+	# Possibly worth examining more closely later
+	assert np.allclose([v for v in agg.agg.test.values],1.4999,rtol=1e-4)
+
 
 def test_aggregate_with_weights(ds=ds):
 	# Create polygon covering multiple pixels
