@@ -1,5 +1,7 @@
-from .export import (prep_for_nc,prep_for_csv,output_data)
+from .export import (prep_for_nc,prep_for_csv,output_data,export_weightmap)
 import warnings
+import os
+import re
 
 try:
     import cartopy 
@@ -11,6 +13,12 @@ try:
 except ImportError:
     no_plotting = True
 
+try:
+    import tables
+except ImportError:
+    no_hd5_output = True
+
+
 # POSSIBLE CHANGE: I'm not quite sure how python deals with memory
 # in this case - but [aggregated], which contains [ds] as one of its
 # fields (for keeping the right variables / shapes upon export) may
@@ -24,11 +32,12 @@ class weightmap(object):
     """ Class for mapping from pixels to polgyons, output from :func:`xagg.wrappers.pixel_overlaps`
     
     """
-    def __init__(self,agg,source_grid,geometry,weights='nowghts'):
+    def __init__(self,agg,source_grid,geometry,overlap_da=None,weights='nowghts'):
         self.agg = agg
         self.source_grid = source_grid
         self.geometry = geometry
         self.weights = weights
+        self.overlap_da = overlap_da
         
     def diag_fig(self,poly_idx):
         """ (NOT YET IMPLEMENTED) a diagnostic figure of the aggregation
@@ -40,6 +49,12 @@ class weightmap(object):
         #    - shaded by the amount of overlapping area 
         #    - or the total weight (area+weight)
 
+    def to_file(self,fn,overwrite=False):
+        """ Save a copy of the weightmap, to avoid recalculating it
+        """
+        export_weightmap(self,fn,overwrite)
+
+
 
 class aggregated(object):
     """ Class for aggregated data, output from :func:`xagg.core.aggregate`
@@ -52,20 +67,27 @@ class aggregated(object):
         self.weights = weights
     
     # Conversion functions
-    def to_dataset(self,loc_dim='pix_idx'):
+    def to_dataset(self,loc_dim='poly_idx'):
         """ Convert to xarray dataset.
         """
         ds_out = prep_for_nc(self,loc_dim=loc_dim)
         return ds_out
 
-    def to_dataframe(self):
+    def to_geodataframe(self):
+        """ Convert to geopandas geodataframe.
+        """
+        df_out = prep_for_csv(self,add_geom=True)
+        return df_out
+
+    def to_dataframe(self,loc_dim='poly_idx'):
         """ Convert to pandas dataframe.
         """
-        df_out = prep_for_csv(self)
-        return df_out  
+        df_out = prep_for_nc(self,loc_dim=loc_dim)
+        df_out = df_out.to_dataframe()
+        return df_out
 
     # Export functions
-    def to_netcdf(self,fn,loc_dim='pix_idx'):
+    def to_netcdf(self,fn,loc_dim='poly_idx'):
         """ Save as netcdf
         """
         output_data(self,
