@@ -2,11 +2,59 @@ import xarray as xr
 import pandas as pd
 import geopandas as gpd
 import numpy as np
-
+import os
+import re
+import warnings
+import shutil
 
 from . aux import (normalize,fix_ds,get_bnds,subset_find)
 
 
+def export_weightmap(wm_obj,fn,overwrite=False):
+    """ Save a copy of the weightmap, to avoid recalculating it
+    """
+    ###### NEED TO SETUP HD5 DEPENDENCY? THROUGH OPTIONAL DEPENDENCY MAYBE? 
+    warnings.warn('export_weightmap() is still an experimental feature. use with care.')
+
+
+    if (not overwrite) and (os.path.exists(fn)):
+        raise FileExistsError(fn+'/ already exists. Either change the [fn] or set overwrite=True.')
+
+    try: 
+        ###### Save geometry
+        # This also creates a folder, into which we can add the rest
+        # of the files
+        wm_obj.geometry.to_file(fn)
+
+        ####### Save agg 
+        # Turn into a dataframe from a geodataframe; only dataframes
+        # can be converted to HD5 files (and there's no geographic 
+        # information here that's needed; can all be saved in the 
+        # geographic save below)
+        df_out = pd.DataFrame(wm_obj.agg)
+        df_out.to_hdf(fn+'/'+re.split('\/',fn)[-1]+'.h5','wm')
+
+        ###### Save source grid
+        for k in wm_obj.source_grid:
+            # Adding v to variable (i.e., 'latv', 'lonv') needed 
+            # because of the multi-index being used here 
+            wm_obj.source_grid[k].reset_index('loc').to_dataset(name=k+'v').to_netcdf(fn+'/'+re.split('\/',fn)[-1]+'_'+k+'.nc')
+
+        ###### Save weights
+        # (If weights == 'nowghts', then no file to load)
+        if (type(wm_obj.weights) is not str) or (wm_obj.weights != 'nowghts'):
+            # Setting astype(object) to make sure integral weights
+            # don't change the general type of the frame. This may
+            # only affect the testing routines, but setting this here
+            # to be explicit 
+            wm_obj.weights.astype(object).to_csv(fn+'/'+re.split('\/',fn)[-1]+'_weights.csv')
+    except:
+        # Remove files that have already been generated, to make 
+        # sure no flawed files are floating around. 
+        shutil.rmtree(fn)
+        raise Exception('Error while saving. The directory '+fn+' has been removed.')
+
+
 def prep_for_nc(agg_obj,loc_dim='poly_idx'):
     """ Preps aggregated data for output as a netcdf
     
@@ -75,7 +123,7 @@ def prep_for_nc(agg_obj,loc_dim='poly_idx'):
     return ds_out
 
 
-def prep_for_csv(agg_obj):
+def prep_for_csv(agg_obj,add_geom=False):
     """ Preps aggregated data for output as a csv
     
     Concretely, aggregated data is placed in a new pandas dataframe
@@ -94,6 +142,12 @@ def prep_for_csv(agg_obj):
     ----------------
     agg_obj : :class:`xagg.classes.aggregated`
         the output from :func:`aggregate`
+
+    add_geom : bool, default `False`
+        if `True`, adds the geometry from the original shapefile/
+        geodataframe back into the prepped output; this is for 
+        the `.to_geodataframe()` conversion option. 
+
                
     Returns
     ----------------
@@ -134,6 +188,10 @@ def prep_for_csv(agg_obj):
                              expanded_var],
                             axis=1)
         del expanded_var
+
+    if add_geom:
+        # Return the geometry from the original geopandas.GeoDataFrame
+        csv_out.geometry = agg_obj.geometry
         
     # Return 
     return csv_out
@@ -155,7 +213,8 @@ def output_data(agg_obj,output_format,output_fn,loc_dim='poly_idx'):
     loc_dim : str, optional. default = ``'poly_idx'``
         the name of the
         dimension with location indices; used
-        only by :func:`xagg.export.prep_for_nc`
+        by :func:`xagg.export.prep_for_nc` for nc, 
+        csv output
                      
     Returns
     ---------------
@@ -163,7 +222,7 @@ def output_data(agg_obj,output_format,output_fn,loc_dim='poly_idx'):
         -  "netcdf": the :class:`xarray.Dataset` on which ``.to_netcdf``
            was called
         -  "csv": the :class:`pandas.Dataframe` on which ``.to_csv`` 
-           was called
+           was called (uses the `xarray` `ds.to_dataframe()` functionality)
         - "shp": the :class:`geopandas.GeoDataDrame` on which ``.to_file`` was called
      
     """
@@ -183,7 +242,8 @@ def output_data(agg_obj,output_format,output_fn,loc_dim='poly_idx'):
 
     elif output_format == 'csv':
 
-        csv_out = prep_for_csv(agg_obj)
+        csv_out = prep_for_nc(agg_obj,loc_dim=loc_dim)
+        csv_out = csv_out.to_dataframe()
 
         # Save as csv
         if not output_fn.endswith('.csv'):
